""" 自然言語理解APIの雑談ドメインを拡張するサンプル
https://developer.yahoo.co.jp/webapi/jlp/nlu/v1/doc/domain/say.html
"""

from abc import ABCMeta, abstractmethod
import os
import sys

import yj.nlu.api


class Generator(metaclass=ABCMeta):
    @abstractmethod
    def match(self, nlu_result):
        pass

    @abstractmethod
    def generate_response(self, nlu_result):
        pass


class ChatGenerator(Generator):
    RESPONSES = {"AGENT_AGE": "年齢は秘密です。", "AGENT_BIRTHDATE": "誕生日は秘密です。",
                 "GREETINGS_BYE": "また話しかけてくださいね。", "GREETINGS_GOOD_MORNING": "おはようございます。お話できて嬉しいです。"}

    def match(self, nlu_result):
        " 自然言語理解APIが推定した、発話の意図 "
        domain = nlu_result["method"]
        " 発話の意図が雑談 (SAY) の時のみTrueを返す "
        return domain == "SAY"

    def generate_response(self, nlu_result):
        if "param_small_talk_intent" in nlu_result:
            small_talk_intent = nlu_result["param_small_talk_intent"]
            if small_talk_intent in ChatGenerator.RESPONSES:
                response = ChatGenerator.RESPONSES[small_talk_intent]
                return response

        if "param_text" not in nlu_result:
            raise ValueError("param_text not found")
        response = nlu_result["param_text"]

        return response


class Assistant(object):
    def __init__(self, nlu):
        self.nlu = nlu
        self.generator = ChatGenerator()

    def talk(self, utterance):
        response = self.nlu.analyze(utterance)
        result = response["result"]
        if self.generator.match(result):
            return self.generator.generate_response(result)
        return "すみません。よくわかりませんでした。"


def main():
    nlu_appid = os.getenv("NLU_API_APPID", "your NLU appid")
    nlu = yj.nlu.api.NluApi(appid=nlu_appid)
    assistant = Assistant(nlu)

    utterance = sys.stdin.readline().rstrip()
    while utterance:
        sys.stdout.write("入力:{0}\t応答:".format(utterance))

        response = assistant.talk(utterance)
        sys.stdout.write("{}\n".format(response))

        utterance = sys.stdin.readline().rstrip()


if __name__ == '__main__':
    main()
